@@ -1,11 +1,16 @@
 import { RepositorySpec, SyncOptions } from 'app/api/clients/provisioning/v0alpha1';
 
-import { StatusInfo, RepositoryFormData } from '../types';
+import { AlertAction } from '../Shared/ProvisioningAlert';
+import { RepositoryFormData, StatusInfo } from '../types';
 
-export type WizardStep = 'connection' | 'bootstrap' | 'finish' | 'synchronize';
+export type WizardStep = 'authType' | 'githubApp' | 'connection' | 'bootstrap' | 'finish' | 'synchronize';
 
 export type RepoType = RepositorySpec['type'];
 
+export type GitHubAuthType = 'pat' | 'github-app';
+
+export type GitHubAppMode = 'existing' | 'new';
+
 export interface MigrateFormData {
   history: boolean;
   identifier: boolean;
@@ -16,6 +21,11 @@ export interface WizardFormData {
   repository: RepositoryFormData;
   migrate?: MigrateFormData;
   repositoryName?: string;
+  githubAuthType?: GitHubAuthType;
+  githubAppMode?: GitHubAppMode;
+  githubApp?: {
+    connectionName?: string;
+  };
 }
 
 export type Target = SyncOptions['target'];
@@ -40,7 +50,9 @@ export const RepoTypeDisplay: { [key in RepoType]: string } = {
 export type StepStatusInfo =
   | { status: 'idle' | 'running' }
   | { status: 'success'; success?: string | StatusInfo }
-  | { status: 'error'; error: string | StatusInfo }
+  | { status: 'error'; error: string | StatusInfo; action?: AlertAction }
   | { status: 'warning'; warning: string | StatusInfo };
 
+export type ConnectionCreationResult = { success: true; connectionName: string } | { success: false; error: string };
+
 export type InstructionAvailability = Extract<RepoType, 'bitbucket' | 'gitlab' | 'github'>;
